@@ -1,7 +1,7 @@
 // This module defines the "official" high-level API of the Qt port of
 // Scintilla.
 //
-// Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2010 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
@@ -25,11 +25,6 @@
 // http://trolltech.com/products/qt/licenses/licensing/licensingoverview
 // or contact the sales department at sales@riverbankcomputing.com.
 // 
-// This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
-// INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
-// A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
-// granted herein.
-// 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
@@ -53,8 +48,13 @@ extern "C++" {
 #include <Qsci/qsciscintillabase.h>
 
 
-class QsciLexer;
+class QIODevice;
+class QPoint;
+
 class QsciCommandSet;
+class QsciLexer;
+class QsciStyle;
+class QsciStyledText;
 class ListBoxQt;
 
 
@@ -88,6 +88,18 @@ public:
         AiClosing = 0x04
     };
 
+    //! This enum defines the different annotation display styles.
+    enum AnnotationDisplay {
+        //! Annotations are not displayed.
+        AnnotationHidden = ANNOTATION_HIDDEN,
+
+        //! Annotations are drawn left justified with no adornment.
+        AnnotationStandard = ANNOTATION_STANDARD,
+
+        //! Annotations are surrounded by a box.
+        AnnotationBoxed = ANNOTATION_BOXED
+    };
+
     //! This enum defines the different sources for auto-completion lists.
     enum AutoCompletionSource {
         //! No sources are used, ie. automatic auto-completion is disabled.
@@ -191,6 +203,29 @@ public:
         BoxedTreeFoldStyle
     };
 
+    //! This enum defined the different margin types.
+    enum MarginType {
+        //! The margin contains symbols, including those used for folding.
+        SymbolMargin = SC_MARGIN_SYMBOL,
+
+        //! The margin contains symbols and uses the default foreground color
+        //! as its background color.
+        SymbolMarginDefaultForegroundColor = SC_MARGIN_FORE,
+
+        //! The margin contains symbols and uses the default background color
+        //! as its background color.
+        SymbolMarginDefaultBackgroundColor = SC_MARGIN_BACK,
+
+        //! The margin contains line numbers.
+        NumberMargin = SC_MARGIN_NUMBER,
+
+        //! The margin contains styled text.
+        TextMargin = SC_MARGIN_TEXT,
+
+        //! The margin contains right justified styled text.
+        TextMarginRightJustified = SC_MARGIN_RTEXT
+    };
+
     //! This enum defines the different pre-defined marker symbols.
     enum MarkerSymbol {
         //! A circle.
@@ -332,6 +367,32 @@ public:
     virtual QStringList apiContext(int pos, int &context_start,
             int &last_word_start);
 
+    //! Annotate the line \a line with the text \a text using the style number
+    //! \a style.
+    void annotate(int line, const QString &text, int style);
+
+    //! Annotate the line \a line with the text \a text using the style \a
+    //! style.
+    void annotate(int line, const QString &text, const QsciStyle &style);
+
+    //! Annotate the line \a line with the styled text \a text.
+    void annotate(int line, const QsciStyledText &text);
+
+    //! Annotate the line \a line with the list of styled text \a text.
+    void annotate(int line, const QValueList<QsciStyledText> &text);
+
+    //! Returns the annotation on line \a line, if any.
+    QString annotation(int line) const;
+
+    //! Returns the display style for annotations.
+    //!
+    //! \sa setAnnotationDisplay()
+    AnnotationDisplay annotationDisplay() const;
+
+    //! The annotations on line \a line are removed.  If \a line is negative
+    //! then all annotations are removed.
+    void clearAnnotations(int line = -1);
+
     //! Returns true if auto-completion lists are case sensitive.
     //!
     //! \sa setAutoCompletionCaseSensitivity()
@@ -405,6 +466,12 @@ public:
     //! is no current lexer then true is returned.
     bool caseSensitive() const;
 
+    //! Clear all current folds, i.e. ensure that all lines are displayed
+    //! unfolded.
+    //!
+    //! \sa setFolding()
+    void clearFolds();
+
     //! Clear all registered images.
     //!
     //! \sa registerImage()
@@ -592,6 +659,15 @@ public:
     //! at that position.
     int lineAt(const QPoint &pos) const;
 
+    //! QScintilla uses the combination of a line number and a character index
+    //! from the start of that line to specify the position of a character
+    //! within the text.  The underlying Scintilla instead uses a byte index
+    //! from the start of the text.  This will convert the \a position byte
+    //! index to the \a *line line number and \a *index character index.
+    //!
+    //! \sa positionFromLineIndex()
+    void lineIndexFromPosition(int position, int *line, int *index) const;
+
     //! Returns the length of line \a line int bytes or -1 if there is no such
     //! line.  In order to get the length in characters use text(line).length().
     int lineLength(int line) const;
@@ -611,7 +687,7 @@ public:
 
     //! Returns true if line numbers are enabled for margin \a margin.
     //!
-    //! \sa setMarginLineNumbers(), SCI_GETMARGINTYPEN
+    //! \sa setMarginLineNumbers(), marginType(), SCI_GETMARGINTYPEN
     bool marginLineNumbers(int margin) const;
 
     //! Returns the marker mask of margin \a margin.
@@ -624,47 +700,53 @@ public:
     //! \sa setMarginSensitivity(), marginClicked(), SCI_GETMARGINTYPEN
     bool marginSensitivity(int margin) const;
 
+    //! Returns the type of margin \a margin.
+    //!
+    //! \sa setMarginType(), SCI_GETMARGINTYPEN
+    MarginType marginType(int margin) const;
+
     //! Returns the width in pixels of margin \a margin.
     //!
     //! \sa setMarginWidth(), SCI_GETMARGINWIDTHN
     int marginWidth(int margin) const;
 
-    //! Define a marker using the symbol \a sym with the marker number \a mnr.
-    //! If \a mnr is -1 then the marker number is automatically allocated.  The
-    //! marker number is returned or -1 if the marker number was already
-    //! allocated or too many markers have been defined.
+    //! Define a type of marker using the symbol \a sym with the marker number
+    //! \a mnr.  If \a mnr is -1 then the marker number is automatically
+    //! allocated.  The marker number is returned or -1 if too many types of
+    //! marker have been defined.
     //!
-    //! Markers are small geometric symbols and character used, for example, to
-    //! indicate the current line or, in debuggers, to indicate breakpoints.
+    //! Markers are small geometric symbols and characters used, for example,
+    //! to indicate the current line or, in debuggers, to indicate breakpoints.
     //! If a margin has a width of 0 then its markers are not drawn, but their
     //! background colours affect the background colour of the corresponding
     //! line of text.
     //!
-    //! There may be up to 32 markers defined and each line of text has a set
-    //! of these markers associated with it.  Markers are drawn according to
-    //! their numerical identifier.  Markers try to move with their text by
-    //! tracking where the start of their line moves to.  For example, when a
-    //! line is deleted its markers are added to previous line's markers.
+    //! There may be up to 32 types of marker defined at a time and each line
+    //! of text has a set of marker instances associated with it.  Markers are
+    //! drawn according to their numerical identifier.  Markers try to move
+    //! with their text by tracking where the start of their line moves to.
+    //! For example, when a line is deleted its markers are added to previous
+    //! line's markers.
     //!
-    //! Each marker is identified by a marker number.  Each instance of a
+    //! Each marker type is identified by a marker number.  Each instance of a
     //! marker is identified by a marker handle.
     int markerDefine(MarkerSymbol sym, int mnr = -1);
 
     //! Define a marker using the character \a ch with the marker number
     //! \a mnr.  If \a mnr is -1 then the marker number is automatically
-    //! allocated.  The marker number is returned or -1 if the marker number
-    //! was already allocated or too many markers have been defined.
+    //! allocated.  The marker number is returned or -1 if too many markers
+    //! have been defined.
     int markerDefine(char ch, int mnr = -1);
 
     //! Define a marker using a copy of the pixmap \a pm with the marker number
     //! \a mnr.  If \a mnr is -1 then the marker number is automatically
-    //! allocated.  The marker number is returned or -1 if the marker number
-    //! was already allocated or too many markers have been defined.
+    //! allocated.  The marker number is returned or -1 if too many markers
+    //! have been defined.
     int markerDefine(const QPixmap &pm, int mnr = -1);
 
-    //! Add a marker number \a mnr to line number \a linenr.  A handle for the
-    //! marker is returned which can be used to track the marker's position, or
-    //! -1 if the \a mnr was invalid.
+    //! Add an instance of marker number \a mnr to line number \a linenr.  A
+    //! handle for the marker is returned which can be used to track the
+    //! marker's position, or -1 if the \a mnr was invalid.
     //!
     //! \sa markerDelete(), markerDeleteAll(), markerDeleteHandle()
     int markerAdd(int linenr, int mnr);
@@ -714,6 +796,21 @@ public:
     //! \sa setPaper()
     QColor paper() const;
 
+    //! QScintilla uses the combination of a line number and a character index
+    //! from the start of that line to specify the position of a character
+    //! within the text.  The underlying Scintilla instead uses a byte index
+    //! from the start of the text.  This will return the byte index
+    //! corresponding to the \a line line number and \a index character index.
+    //!
+    //! \sa lineIndexFromPosition()
+    int positionFromLineIndex(int line, int index) const;
+
+    //! Reads the current document from the \a io device and returns true if
+    //! there was no error.
+    //!
+    //! \sa write()
+    bool read(QIODevice *io);
+
     //! Recolours the document between the \a start and \a end positions.
     //! \a start defaults to the start of the document and \a end defaults to
     //! the end of the document.
@@ -742,6 +839,12 @@ public:
     //! \sa resetFoldMarginColors()
     void setFoldMarginColors(const QColor &fore, const QColor &back);
 
+    //! Set the display style for annotations.  The default is
+    //! AnnotationStandard.
+    //!
+    //! \sa annotationDisplay()
+    void setAnnotationDisplay(AnnotationDisplay display);
+
     //! Enable the use of fill-up characters, either those explicitly set or
     //! those set by a lexer.  By default, fill-up characters are disabled.
     //!
@@ -815,6 +918,30 @@ public:
     //! \sa edgeMode()
     void setEdgeMode(EdgeMode mode);
 
+    //! Set the margin text of line \a line with the text \a text using the
+    //! style number \a style.
+    void setMarginText(int line, const QString &text, int style);
+
+    //! Set the margin text of line \a line with the text \a text using the
+    //! style \a style.
+    void setMarginText(int line, const QString &text, const QsciStyle &style);
+
+    //! Set the margin text of line \a line with the styled text \a text.
+    void setMarginText(int line, const QsciStyledText &text);
+
+    //! Set the margin text of line \a line with the list of styled text \a
+    //! text.
+    void setMarginText(int line, const QValueList<QsciStyledText> &text);
+
+    //! Set the type of margin \a margin to type \a type.
+    //!
+    //! \sa marginType(), SCI_SETMARGINTYPEN
+    void setMarginType(int margin, MarginType type);
+
+    //! The margin text on line \a line is removed.  If \a line is negative
+    //! then all margin text is removed.
+    void clearMarginText(int line = -1);
+
     //! Set the background colour, including the alpha component, of marker
     //! \a mnr to \a col.  If \a mnr is -1 then the colour of all markers is
     //! set.  The default is white.
@@ -900,7 +1027,7 @@ public:
     //! \sa setTabWidth()
     int tabWidth() const;
 
-    //! Returns the text edit's text.
+    //! Returns the text of the current document.
     //!
     //! \sa setText()
     QString text() const;
@@ -920,6 +1047,9 @@ public:
     //! \sa setWhitespaceVisibility()
     WhitespaceVisibility whitespaceVisibility() const;
 
+    //! Returns the word at the \a point screen coordinates.
+    QString wordAtPoint(const QPoint &point) const;
+
     //! Returns the set of valid word character as defined by the current
     //! language lexer.  If there is no current lexer then the set contains an
     //! an underscore, numbers and all upper and lower case alphabetic
@@ -933,6 +1063,12 @@ public:
     //! \sa setWrapMode()
     WrapMode wrapMode() const;
 
+    //! Writes the current document to the \a io device and returns true if
+    //! there was no error.
+    //!
+    //! \sa read()
+    bool write(QIODevice *io) const;
+
 public slots:
     //! Appends the text \a text to the end of the text edit.  Note that the
     //! undo/redo history is cleared by this function.
@@ -1206,7 +1342,7 @@ public slots:
     //! Enables or disables, according to \a lnrs, the display of line numbers
     //! in margin \a margin.
     //!
-    //! \sa marginLineNumbers(), SCI_SETMARGINTYPEN
+    //! \sa marginLineNumbers(), setMarginType(), SCI_SETMARGINTYPEN
     virtual void setMarginLineNumbers(int margin, bool lnrs);
 
     //! Sets the marker mask of margin \a margin to \a mask.  Only those
@@ -1254,7 +1390,7 @@ public slots:
 
     //! Sets the selection which starts at position \a indexFrom in line
     //! \a lineFrom and ends at position \a indexTo in line \a lineTo.  The
-    //! cursor is moved to the end of the selection.
+    //! cursor is moved to position \a indexTo in \a lineTo.
     //!
     //! \sa getSelection()
     virtual void setSelection(int lineFrom, int indexFrom, int lineTo,
@@ -1413,7 +1549,8 @@ private slots:
     void handleCharAdded(int charadded);
     void handleMarginClick(int pos, int margin, int modifiers);
     void handleModified(int pos, int mtype, const char *text, int len,
-            int added, int line, int foldNow, int foldPrev);
+            int added, int line, int foldNow, int foldPrev, int token,
+            int annotationLinesAdded);
     void handlePropertyChange(const char *prop, const char *val);
     void handleSavePointReached();
     void handleSavePointLeft();
@@ -1429,6 +1566,10 @@ private slots:
     void handleUpdateUI();
 
 private:
+    typedef QCString ScintillaString;
+
+    void detachLexer();
+
     enum IndentState {
         isNone,
         isKeywordStart,
@@ -1445,8 +1586,6 @@ private:
     int findStyledWord(const char *text, int style, const char *words);
 
     void checkMarker(int &mnr);
-    long posFromLineIndex(int line, int index) const;
-    void lineIndexFromPos(long pos, int *line, int *index) const;
     int currentIndent() const;
     int indentWidth() const;
     bool doFind();
@@ -1457,6 +1596,7 @@ private:
             int visLevels = 0, int level = -1);
     void setFoldMarker(int marknr, int mark = SC_MARK_EMPTY);
     QString convertTextS2Q(const char *s) const;
+    ScintillaString convertTextQ2S(const QString &q) const;
     void setLexerStyle(int style);
     void setStylesFont(const QFont &f, int style);
 
@@ -1475,7 +1615,10 @@ private:
     bool isStartChar(char ch) const;
 
     bool ensureRW();
-    void insertAtPos(const QString &text, long pos);
+    void insertAtPos(const QString &text, int pos);
+
+    ScintillaString styleText(const QValueList<QsciStyledText> &styled_text,
+            char **styles, int style_offset = 0);
 
     struct FindState
     {
@@ -1498,6 +1641,7 @@ private:
     int ctPos;
     bool selText;
     FoldStyle fold;
+    int foldmargin;
     bool autoInd;
     BraceMatch braceMode;
     AutoCompletionSource acSource;
@@ -1513,7 +1657,6 @@ private:
     QGuardedPtr<QsciLexer> lex;
     QsciCommandSet *stdCmds;
     QsciDocument doc;
-    bool modified;
     QColor nl_text_colour;
     QColor nl_paper_colour;
     QCString explicit_fillups;
